export declare type Task<C = unknown, T = unknown> = (context: C) => T | Promise<T>;
export declare type ExecutionResult<T = unknown> = {
    type: 'SUCCESS';
    returnValue: T;
} | {
    type: 'ERROR';
    caughtValue: any;
};
export interface ExecutionMetadata {
    /**
     * Start time of the first attempt.
     */
    firstAttemptStartTime: Date;
    /**
     * End time of the first attempt.
     */
    firstAttemptEndTime: Date;
    /**
     * The attempt number of this attempt. `1` for first attempt. `2` for second retry attempt.
     */
    attemptNumber: number;
    /**
     * Start time of this attempt.
     */
    startTime: Date;
    /**
     * End time of this attempt.
     */
    endTime: Date;
}
export interface NextRunRequest {
    startDelayOrTime: number | Date;
    /**
     * Whether the next run is a retry attempt
     */
    isRetry: boolean;
}
export declare type NextRunTimeEvaluator<C = unknown, T = unknown> = (result: ExecutionResult<T>, meta: ExecutionMetadata, context: C) => NextRunRequest | null;
declare type CParamsWithoutContext<C, T> = [
    task: Task<C, T>,
    nextRunTime?: NextRunTimeOptions | NextRunTimeEvaluator<C, T> | null
];
declare type CParamsWithContext<C, T> = [
    task: Task<C, T>,
    nextRunTime: NextRunTimeOptions | NextRunTimeEvaluator<C, T> | null | undefined,
    initialContext: C
];
/**
 * A single instance task scheduler with flexible next run time.
 *
 * Stability: 1 - Experimental.
 */
export declare class SingleInstanceTaskScheduler<C = undefined, T = unknown> {
    #private;
    constructor(...values: undefined extends C ? CParamsWithoutContext<C, T> : CParamsWithContext<C, T>);
    /**
     * Whether a next run is scheduled.
     */
    get scheduled(): boolean;
    /**
     * The next run time of the task. `null` indicates that there will be no next run.
     * `undefined` indicates that the next run time is going to be known after the current
     * task end.
     */
    get nextRunTime(): Date | null | undefined;
    /**
     * Whether the task is currently running.
     */
    get running(): boolean;
    /**
     * Set options for next run time, or a function that return the next run time of the task.
     * Scheduled next run time will not be changed.
     *
     * If a function is provided, it will be called after a task ended to evaluate the
     * next run time. The returned value can be a delay in milliseconds, or an absolute
     * date time `Date` object, or `null` which indicate no next run. Default is `null`.
     */
    setNextRunTimeOptions(value: NextRunTimeOptions | NextRunTimeEvaluator<C, T> | null): void;
    /**
     * Schedule the task to run after a given milliseconds or absolute date time. If
     * the task is already scheduled, it will be re-scheduled.
     * @param startDelayOrTime Start time of the next run. A delay in milliseconds, or an absolute
     *   date time.
     */
    schedule(startDelayOrTime: number | Date): void;
    /**
     * Cancel the next scheduled run. Running task will not be affected.
     */
    cancelNextRun(): void;
    private scheduleWithResult;
    /**
     * Run the scheduled task immediately without waiting for the task returned value. Next
     * run will be scheduled if configured.
     */
    run(): void;
    /**
     * Run the scheduled task immediately and return a promise which will be fulfilled
     * with the task returned value. Next run will be scheduled if configured.
     */
    runReturnResult(): Promise<T>;
}
export interface OneTimeEvaluateOptions {
    type: 'ONE_TIME';
}
export interface IntervalEvaluateOptions {
    type: 'RUN_START_TIME' | 'RUN_END_TIME';
    delay: number;
}
export interface OnErrorEvaluateOptions {
    delay: number;
    /**
     * Default is `Infinity`.
     */
    attempt?: number;
}
export interface NextRunTimeOptions {
    onSuccess: OneTimeEvaluateOptions | IntervalEvaluateOptions;
    /**
     * Default is `undefined`, which will not perform retry.
     */
    onError?: OnErrorEvaluateOptions;
}
export declare function _buildEvaluator<C, T>(options: NextRunTimeOptions): NextRunTimeEvaluator<C, T>;
export {};
